package user

import (
	"context"
	"crypto/md5"
	"encoding/hex"

	"github.com/jackc/pgx/v5"
	"github.com/jackc/pgx/v5/pgxpool"
)

type UserEntity struct {
	Id       int
	Username string
	Password string
}

type UsernameAlreadyExistsErr struct{}

func (e *UsernameAlreadyExistsErr) Error() string {
	return "username already exists error"
}

type UserNotFoundErr struct{}

func (e *UserNotFoundErr) Error() string {
	return "user not found error"
}

type UserService struct {
	db *pgxpool.Pool
}

func NewUserService(
	db *pgxpool.Pool,
) *UserService {
	return &UserService{
		db: db,
	}
}

func (s *UserService) Login(ctx context.Context, user *UserEntity) (*UserEntity, error) {
	args := pgx.NamedArgs{
		"username": user.Username,
		"password": hashPassword(user.Username, user.Password, "crab"),
	}

	query := `SELECT id FROM users WHERE username = @username AND password = @password LIMIT 1`
	if err := s.db.QueryRow(ctx, query, args).Scan(&user.Id); err != nil {
		query := `INSERT INTO users (username, password) VALUES (@username, @password) RETURNING id`
		if err := s.db.QueryRow(ctx, query, args).Scan(&user.Id); err != nil {
			return user, err
		}
	}

	return user, nil
}

func hashPassword(username string, password string, salt string) string {
	return getMD5Hash(username + password + salt)
}

func getMD5Hash(text string) string {
	hash := md5.Sum([]byte(text))
	return hex.EncodeToString(hash[:])
}
